using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Threading;
using System.Collections;
using System.Windows.Controls.Primitives;
using System.Data;
using System.Windows.Threading;
using System.ComponentModel;
using System.Collections.ObjectModel;
using Microsoft.Windows.Controls;


namespace DataGridSortDemo
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : System.Windows.Window
    {
        public Window1()
        {
            InitializeComponent();
        }

        private bool UseCustomSort
        {
            get { return CustomSortCB.IsChecked == true; }
        }

        private void WPF_DataGrid_Sorting(object sender, DataGridSortingEventArgs e)
        {
            if (_stopwatch == null)
            {
                _stopwatch = new Stopwatch();
            }
            else
            {
                _stopwatch.Reset();
            }

            _stopwatch.Start();

            if (UseCustomSort)
            {
                Console.WriteLine("WPF_DataGrid_Sorting ==> Cancel Built in sort...");
                e.Handled = true;   // prevent the built-in sort from sorting

                PerformCustomSort(e.Column);
                CustomSortingDone();
            }
            else
            {
                Dispatcher.BeginInvoke(DispatcherPriority.Normal, new NoArgDelegate(StandardSortingDone));
            }
        }

        private void StandardSortingDone()
        {
            _stopwatch.Stop();
            Console.WriteLine("*Buit-in Sort:={0}", _stopwatch.ElapsedMilliseconds.ToString());
            statusTextBlock1.Text = "Buit-in Sort: " + _stopwatch.ElapsedMilliseconds.ToString() + " msec";
            statusTextBlock1.FontWeight = FontWeights.Bold;
            statusTextBlock2.FontWeight = FontWeights.Normal;
        }

        private void CustomSortingDone()
        {
            _stopwatch.Stop();
            Console.WriteLine("=Custom Sort={0}", _stopwatch.ElapsedMilliseconds.ToString());
            statusTextBlock2.Text = "Custom Sort: " + _stopwatch.ElapsedMilliseconds.ToString() + " msec";
            statusTextBlock2.FontWeight = FontWeights.Bold;
            statusTextBlock1.FontWeight = FontWeights.Normal;
        }

        private void PerformCustomSort(DataGridColumn column)
        {
            Console.WriteLine("In PerformCustomSort...");

            ListSortDirection direction = (column.SortDirection != ListSortDirection.Ascending) ? ListSortDirection.Ascending : ListSortDirection.Descending;
            column.SortDirection = direction;

            ListCollectionView lcv = (ListCollectionView)CollectionViewSource.GetDefaultView(WPF_DataGrid.ItemsSource);
            MySort mySort = new MySort(direction, column);
            lcv.CustomSort = mySort;
        }

        public class MySort : IComparer
        {
            public delegate int TwoArgDelegate(Employee arg1, Employee arg2);
            TwoArgDelegate myCompare;
            public MySort(ListSortDirection direction, DataGridColumn column)
            {
                int dir = (direction == ListSortDirection.Ascending) ? 1 : -1;
                //set a delegate to be called by IComparer.Compare
                switch ((string)column.Header)
                {
                    case "Id":
                        myCompare = (a, b) => { return a.Id.CompareTo(b.Id) * dir; };
                        break;
                    case "Name":
                        myCompare = (a, b) => { return a.Name.CompareTo(b.Name) * dir; };
                        break;
                    case "Position":
                        myCompare = (a, b) => { return a.Position.CompareTo(b.Position) * dir; };
                        break;
                    case "Tel":
                        myCompare = (a, b) => { return a.Telephone.CompareTo(b.Telephone) * dir; };
                        break;
                    case "Email":
                        myCompare = (a, b) => { return a.Email.CompareTo(b.Email) * dir; };
                        break;
                    case "Enabled":
                        myCompare = (a, b) => { return a.Enabled.CompareTo(b.Enabled) * dir; };
                        break;
                    case "City":
                        myCompare = (a, b) => { return a.City.CompareTo(b.City) * dir; };
                        break;
                    case "Country":
                        myCompare = (a, b) => { return a.Country.CompareTo(b.Country) * dir; };
                        break;
                    default:
                        myCompare = (a, b) => { return 0; };
                        break;
                }
            }

            int IComparer.Compare(object X, object Y)
            {
                return myCompare((Employee)X, (Employee)Y);
            }
        }
        private Stopwatch _stopwatch;
    }

    public delegate void NoArgDelegate();

    public class Employees : ObservableCollection<Employee>
    {
        public Employees()
        {
            for (int i = 0; i < 1000; i++)
            {
                Add(new Employee(i * 6 + 1, "Charlie B.", "Developer", "(425)-555-8080", String.Format("{0}", i * 6 + 1) + "@contoso.com", true, "Boston", "USA"));
                Add(new Employee(i * 6 + 2, "Betty Y.", "Developer", "(425)-555-8081", String.Format("{0}", i * 6 + 2) + "@contoso.com", true, "New York", "USA"));
                Add(new Employee(i * 6 + 3, "Luke S.", "Developer", "(425)-555-8082", String.Format("{0}", i * 6 + 3) + "@contoso.com", false, "Rome", "Italy"));
                Add(new Employee(i * 6 + 4, "Kevin U.", "Lead Developer", "(425)-555-8083", String.Format("{0}", i * 6 + 4) + "@contoso.com", false, "Riverside", "USA"));
                Add(new Employee(i * 6 + 5, "Homer S.", "Tester", "(425)-555-8084", String.Format("{0}", i * 6 + 5) + "@contoso.com", true, "Paris", "France"));
                Add(new Employee(i * 6 + 6, "George C.", "Program Manager", "(425)-555-8085", String.Format("{0}", i * 6 + 6) + "@contoso.com", false, "Oslo", "Norway"));
            }
        }
    }

    public class Employee
    {
        public Employee()
        {
            _id = 0;
            _name = "Unknown";
            _position = "N/A";
            _telephone = "(425)-555-0000";
            _email = "unknown@contoso.com";
            _enabled = true;
            _city = "N/A";
            _country = "N/A";
        }

        public Employee(int id, string name, string position, string telephone, string email, bool enabled, string city, string country)
        {
            _id = id;
            _name = name;
            _position = position;
            _telephone = telephone;
            _email = email;
            _enabled = enabled;
            _city = city;
            _country = country;
        }

        public int Id
        {
            get { return _id; }
            set { _id = value; }
        }

        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        public string Position
        {
            get { return _position; }
            set { _position = value; }
        }

        private string _telephone;

        public string Telephone
        {
            get { return _telephone; }
            set { _telephone = value; }
        }

        private string _email;

        public string Email
        {
            get { return _email; }
            set { _email = value; }
        }

        private bool _enabled;

        public bool Enabled
        {
            get { return _enabled; }
            set { _enabled = value; }
        }
        public string City
        {
            get { return _city; }
            set { _city = value; }
        }
        public string Country
        {
            get { return _country; }
            set { _country = value; }
        }

        private int _id;
        private string _name;
        private string _position;
        private string _city;
        private string _country;

    }
}